@@ -20,15 +20,7 @@ import warnings
 
 from six.moves.urllib.parse import parse_qs, urlencode
 
-from .common import (
-    PaymentCommon,
-    PaymentResponse,
-    ResponseError,
-    URL,
-    PAID, ERROR, WAITING,
-    force_text,
-    _
-)
+from .common import ERROR, PAID, URL, WAITING, PaymentCommon, PaymentResponse, ResponseError, _, force_text
 
 __all__ = ['Payment']
 
@@ -38,25 +30,26 @@ LOGGER = logging.getLogger(__name__)
 
 
 class Payment(PaymentCommon):
-    '''
-       Dummy implementation of the payment interface.
+    """
+    Dummy implementation of the payment interface.
 
-       It is used with a dummy implementation of a bank payment service that
-       you can find on:
+    It is used with a dummy implementation of a bank payment service that
+    you can find on:
 
-           http://dummy-payment.demo.entrouvert.com/
+        http://dummy-payment.demo.entrouvert.com/
+
+    You must pass the following keys inside the options dictionnary:
+     - dummy_service_url, the URL of the dummy payment service, it defaults
+       to the one operated by Entr'ouvert.
+     - automatic_return_url: where to POST to notify the service of a
+       payment
+     - origin: a human string to display to the user about the origin of
+       the request.
+     - siret: an identifier for the eCommerce site, fake.
+     - normal_return_url: the return URL for the user (can be overriden on a
+       per request basis).
+    """
 
-       You must pass the following keys inside the options dictionnary:
-        - dummy_service_url, the URL of the dummy payment service, it defaults
-          to the one operated by Entr'ouvert.
-        - automatic_return_url: where to POST to notify the service of a
-          payment
-        - origin: a human string to display to the user about the origin of
-          the request.
-        - siret: an identifier for the eCommerce site, fake.
-        - normal_return_url: the return URL for the user (can be overriden on a
-          per request basis).
-    '''
     description = {
         'caption': 'Dummy payment backend',
         'parameters': [
@@ -79,8 +72,7 @@ class Payment(PaymentCommon):
             },
             {
                 'name': 'origin',
-                'caption': _('name of the requesting service, '
-                           'to present in the user interface'),
+                'caption': _('name of the requesting service, ' 'to present in the user interface'),
                 'type': str,
                 'default': 'origin',
             },
@@ -129,23 +121,51 @@ class Payment(PaymentCommon):
         ],
     }
 
-    def request(self, amount, name=None, address=None, email=None, phone=None,
-                orderid=None, info1=None, info2=None, info3=None,
-                next_url=None, capture_day=None, subject=None, **kwargs):
+    def request(
+        self,
+        amount,
+        name=None,
+        address=None,
+        email=None,
+        phone=None,
+        orderid=None,
+        info1=None,
+        info2=None,
+        info3=None,
+        next_url=None,
+        capture_day=None,
+        subject=None,
+        **kwargs,
+    ):
         self.logger.debug(
             '%s amount %s name %s address %s email %s phone %s'
             ' next_url %s info1 %s info2 %s info3 %s kwargs: %s',
-            __name__, amount, name, address, email, phone, info1, info2, info3, next_url, kwargs)
+            __name__,
+            amount,
+            name,
+            address,
+            email,
+            phone,
+            info1,
+            info2,
+            info3,
+            next_url,
+            kwargs,
+        )
         transaction_id = str(uuid.uuid4().hex)
         normal_return_url = self.normal_return_url
         if next_url and not normal_return_url:
-            warnings.warn("passing next_url to request() is deprecated, "
-                          "set normal_return_url in options", DeprecationWarning)
+            warnings.warn(
+                "passing next_url to request() is deprecated, " "set normal_return_url in options",
+                DeprecationWarning,
+            )
             normal_return_url = next_url
         automatic_return_url = self.automatic_return_url
         if self.direct_notification_url and not automatic_return_url:
-            warnings.warn("direct_notification_url option is deprecated, "
-                          "use automatic_return_url", DeprecationWarning)
+            warnings.warn(
+                "direct_notification_url option is deprecated, " "use automatic_return_url",
+                DeprecationWarning,
+            )
             automatic_return_url = self.direct_notification_url
         query = {
             'transaction_id': transaction_id,
@@ -153,11 +173,20 @@ class Payment(PaymentCommon):
             'email': email,
             'return_url': normal_return_url or '',
             'direct_notification_url': automatic_return_url or '',
-            'origin': self.origin
+            'origin': self.origin,
         }
         query.update(
-            dict(name=name, address=address, email=email, phone=phone,
-                 orderid=orderid, info1=info1, info2=info2, info3=info3))
+            dict(
+                name=name,
+                address=address,
+                email=email,
+                phone=phone,
+                orderid=orderid,
+                info1=info1,
+                info2=info2,
+                info3=info3,
+            )
+        )
         if capture_day is not None:
             query['capture_day'] = str(capture_day)
         if subject is not None:
@@ -193,7 +222,8 @@ class Payment(PaymentCommon):
             order_id=transaction_id,
             transaction_id=transaction_id,
             bank_status=form.get('reason'),
-            test=True)
+            test=True,
+        )
         return response
 
     def validate(self, amount, bank_data, **kwargs):
