@@ -14,20 +14,31 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from .common import _
-
 import requests
 from six.moves.urllib.parse import parse_qs, urljoin
 
-from .common import (CANCELLED, ERROR, PAID, URL, PaymentCommon,
-                     PaymentException, PaymentResponse, ResponseError, WAITING,
-                     ACCEPTED)
+from .common import (
+    ACCEPTED,
+    CANCELLED,
+    ERROR,
+    PAID,
+    URL,
+    WAITING,
+    PaymentCommon,
+    PaymentException,
+    PaymentResponse,
+    ResponseError,
+    _,
+)
 
 __all__ = ['Payment']
 
 
 class Payment(PaymentCommon):
     '''Implements Mollie API, see https://docs.mollie.com/reference/v2/.'''
+
+    has_empty_response = True
+
     service_url = 'https://api.mollie.com/v2/'
 
     description = {
@@ -66,9 +77,12 @@ class Payment(PaymentCommon):
 
     def request(self, amount, **kwargs):
         amount = self.clean_amount(amount, cents=False)
+        orderid = kwargs.pop('orderid', None)
+        subject = kwargs.pop('subject', None)
 
-        metadata = {k: v for k, v in kwargs.items()
-                    if k in ('email', 'first_name', 'last_name') and v is not None}
+        metadata = {
+            k: v for k, v in kwargs.items() if k in ('email', 'first_name', 'last_name') and v is not None
+        }
         body = {
             'amount': {
                 'value': amount,
@@ -79,13 +93,18 @@ class Payment(PaymentCommon):
             'metadata': metadata,
             'description': self.description_text,
         }
+        if orderid is not None:
+            body['description'] = orderid
+            metadata['orderid'] = orderid
+        if subject is not None:
+            body['description'] = subject
+            body['subject'] = subject
 
         resp = self.call_endpoint('POST', 'payments', data=body)
 
         return resp['id'], URL, resp['_links']['checkout']['href']
 
-    def response(self, query_string, redirect=False, order_id_hint=None,
-                 order_status_hint=None, **kwargs):
+    def response(self, query_string, redirect=False, order_id_hint=None, order_status_hint=None, **kwargs):
         if redirect:
             if order_status_hint in (PAID, CANCELLED, ERROR):
                 return PaymentResponse(order_id=order_id_hint, result=order_status_hint)
@@ -118,7 +137,7 @@ class Payment(PaymentCommon):
             order_id=payment_id,
             transaction_id=payment_id,
             bank_status=status,
-            test=resp['mode'] == 'test'
+            test=resp['mode'] == 'test',
         )
         return response
 
@@ -133,13 +152,14 @@ class Payment(PaymentCommon):
             result = response.json()
         except ValueError:
             self.logger.debug('received invalid json %r', response.text)
-            raise PaymentException('%s on endpoint "%s" returned invalid JSON: %s' %
-                                   (method, endpoint, response.text))
+            raise PaymentException(
+                '%s on endpoint "%s" returned invalid JSON: %s' % (method, endpoint, response.text)
+            )
         self.logger.debug('received "%s" with status %s', result, response.status_code)
         try:
             response.raise_for_status()
         except requests.exceptions.HTTPError as e:
             raise PaymentException(
-                '%s error on endpoint "%s": %s "%s"' % (method, endpoint, e,
-                                                        result.get('detail', result)))
+                '%s error on endpoint "%s": %s "%s"' % (method, endpoint, e, result.get('detail', result))
+            )
         return result
