@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # eopayment - online payment library
 # Copyright (C) 2011-2020 Entr'ouvert
 #
@@ -16,18 +15,29 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import hashlib
-import string
-import six
+import uuid
+from decimal import ROUND_HALF_UP, Decimal
+
 from six.moves.urllib import parse as urlparse
-from decimal import Decimal, ROUND_HALF_UP
 
 from .common import (
-    PaymentCommon, PaymentResponse, FORM, CANCELLED, PAID,
-    ERROR, Form, DENIED, ACCEPTED, ORDERID_TRANSACTION_SEPARATOR,
-    WAITING, ResponseError, force_byte, force_text, _
+    ACCEPTED,
+    CANCELLED,
+    DENIED,
+    ERROR,
+    FORM,
+    ORDERID_TRANSACTION_SEPARATOR,
+    PAID,
+    WAITING,
+    Form,
+    PaymentCommon,
+    PaymentResponse,
+    ResponseError,
+    _,
+    force_byte,
+    force_text,
 )
 
-
 ENVIRONMENT_TEST = 'TEST'
 ENVIRONMENT_TEST_URL = 'https://secure.ogone.com/ncol/test/orderstandard.asp'
 ENVIRONMENT_PROD = 'PROD'
@@ -457,10 +467,19 @@ class Payment(PaymentCommon):
             },
             {
                 'name': 'language',
-                'caption': _(u'Language'),
+                'caption': _('Language'),
                 'default': 'fr_FR',
                 'choices': (('fr_FR', 'français'),),
             },
+            {
+                'name': 'encoding',
+                'caption': _('Characters encoding'),
+                'default': 'utf-8',
+                'choices': [
+                    ('iso-8859-1', 'Latin1 (ISO-8859-1)'),
+                    ('utf-8', 'Unicode (UTF-8)'),
+                ],
+            },
             {
                 'name': 'hash_algorithm',
                 'caption': 'Algorithme de hachage',
@@ -482,26 +501,31 @@ class Payment(PaymentCommon):
                 'default': 'EUR',
                 'choices': ('EUR',),
             },
-        ]
+        ],
     }
 
-    def sha_sign(self, algo, key, params, keep):
+    def __init__(self, options, logger=None):
+        # retro-compatibility with old default of latin1
+        options.setdefault('encoding', 'iso-8859-1')
+        super().__init__(options, logger=logger)
+
+    def sha_sign(self, algo, key, params, keep, encoding='iso-8859-1'):
         '''Ogone signature algorithm of query string'''
         values = params.items()
         values = [(a.upper(), b) for a, b in values]
         values = sorted(values)
-        values = [u'%s=%s' % (a, b) for a, b in values if a in keep and b]
+        values = ['%s=%s' % (a, b) for a, b in values if a in keep and b]
         tosign = key.join(values)
         tosign += key
-        tosign = force_byte(tosign, encoding='iso-8859-1')
+        tosign = force_byte(tosign, encoding=encoding)
         hashing = getattr(hashlib, algo)
         return hashing(tosign).hexdigest().upper()
 
-    def sha_sign_in(self, params):
-        return self.sha_sign(self.hash_algorithm, self.sha_in, params, SHA_IN_PARAMS)
+    def sha_sign_in(self, params, encoding='iso-8859-1'):
+        return self.sha_sign(self.hash_algorithm, self.sha_in, params, SHA_IN_PARAMS, encoding=encoding)
 
-    def sha_sign_out(self, params):
-        return self.sha_sign(self.hash_algorithm, self.sha_out, params, SHA_OUT_PARAMS)
+    def sha_sign_out(self, params, encoding='iso-8859-1'):
+        return self.sha_sign(self.hash_algorithm, self.sha_out, params, SHA_OUT_PARAMS, encoding=encoding)
 
     def get_request_url(self):
         if self.environment == ENVIRONMENT_TEST:
@@ -510,20 +534,27 @@ class Payment(PaymentCommon):
             return ENVIRONMENT_PROD_URL
         raise NotImplementedError('unknown environment %s' % self.environment)
 
-    def request(self, amount, orderid=None, name=None, email=None,
-                language=None, description=None, **kwargs):
+    def request(
+        self,
+        amount,
+        orderid=None,
+        name=None,
+        email=None,
+        language=None,
+        description=None,
+        transaction_id=None,
+        **kwargs,
+    ):
+        # use complus for transmitting and receiving the transaction_id see
+        # https://epayments-support.ingenico.com/fr/integration/all-sales-channels/integrate-with-e-commerce/guide#variable-feedback-parameters
+        # orderid is now only used for unicity of payments check (it's
+        # garanteed that no payment for the same ORDERID can happen during a 45
+        # days window, see
+        # https://epayments-support.ingenico.com/fr/integration/all-sales-channels/integrate-with-e-commerce/guide#form-parameters)
+        complus = transaction_id or uuid.uuid4().hex
+        if not orderid:
+            orderid = complus
 
-        reference = self.transaction_id(20, string.digits + string.ascii_letters)
-
-        # prepend order id in payment reference
-        if orderid:
-            if len(orderid) > 24:
-                raise ValueError('orderid length exceeds 25 characters')
-            reference = (
-                orderid
-                + ORDERID_TRANSACTION_SEPARATOR
-                + self.transaction_id(29 - len(orderid),
-                                      string.digits + string.ascii_letters))
         language = language or self.language
         # convertir en centimes
         amount = Decimal(amount) * 100
@@ -531,10 +562,11 @@ class Payment(PaymentCommon):
         amount = amount.quantize(Decimal('1.'), rounding=ROUND_HALF_UP)
         params = {
             'AMOUNT': force_text(amount),
-            'ORDERID': reference,
+            'ORDERID': orderid,
             'PSPID': self.pspid,
             'LANGUAGE': language,
             'CURRENCY': self.currency,
+            'COMPLUS': complus,
         }
         if self.normal_return_url:
             params['ACCEPTURL'] = self.normal_return_url
@@ -558,32 +590,33 @@ class Payment(PaymentCommon):
         form = Form(
             url=url,
             method='POST',
-            fields=[{'type': 'hidden',
-                     'name': key,
-                     'value': params[key]} for key in params])
-        return reference, FORM, form
+            fields=[{'type': 'hidden', 'name': key, 'value': params[key]} for key in params],
+        )
+        return complus, FORM, form
 
     def response(self, query_string, **kwargs):
-        if six.PY3:
-            params = urlparse.parse_qs(query_string, True, encoding='iso-8859-1')
-        else:
-            params = urlparse.parse_qs(query_string, True)
-        params = dict((key.upper(), params[key][0]) for key in params)
-        if not set(params) >= set(['ORDERID', 'PAYID', 'STATUS', 'NCERROR']):
+        params = urlparse.parse_qs(query_string, True, encoding=self.encoding)
+        params = {key.upper(): params[key][0] for key in params}
+        if not set(params) >= {'ORDERID', 'PAYID', 'STATUS', 'NCERROR'}:
             raise ResponseError('missing ORDERID, PAYID, STATUS or NCERROR')
 
-        # uniformize iso-8859-1 encoded values
+        # py2: decode binary strings in query-string
         for key in params:
-            params[key] = force_text(params[key], 'iso-8859-1')
-        reference = params['ORDERID']
+            params[key] = force_text(params[key], self.encoding)
+        orderid = params['ORDERID']
+        complus = params.get('COMPLUS')
         transaction_id = params['PAYID']
         status = params['STATUS']
         error = params['NCERROR']
         signed = False
         if self.sha_in:
             signature = params.get('SHASIGN')
-            expected_signature = self.sha_sign_out(params)
-            signed = signature == expected_signature
+            # check signature against both encoding
+            for encoding in ('iso-8859-1', 'utf-8'):
+                expected_signature = self.sha_sign_out(params, encoding=encoding)
+                signed = signature == expected_signature
+                if signed:
+                    break
         if status == '1':
             result = CANCELLED
         elif status == '2':
@@ -599,15 +632,16 @@ class Payment(PaymentCommon):
             # status 91: payment waiting/pending)
             result = WAITING
         else:
-            self.logger.error('response STATUS=%s NCERROR=%s NCERRORPLUS=%s',
-                              status, error, params.get('NCERRORPLUS', ''))
+            self.logger.error(
+                'response STATUS=%s NCERROR=%s NCERRORPLUS=%s', status, error, params.get('NCERRORPLUS', '')
+            )
             result = ERROR
         # extract reference from received order id
-        if ORDERID_TRANSACTION_SEPARATOR in reference:
-            reference, transaction_id = reference.split(ORDERID_TRANSACTION_SEPARATOR, 1)
+
         return PaymentResponse(
             result=result,
             signed=signed,
             bank_data=params,
-            order_id=reference,
-            transaction_id=transaction_id)
+            order_id=complus or orderid,
+            transaction_id=transaction_id,
+        )
