@@ -1,4 +1,3 @@
-# -*- coding: utf-8
 # eopayment - online payment library
 # Copyright (C) 2011-2020 Entr'ouvert
 #
@@ -15,33 +14,40 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import base64
 import codecs
-from collections import OrderedDict
 import datetime
-import logging
 import hashlib
 import hmac
+import logging
 import re
-import requests
 import uuid
+import warnings
+from collections import OrderedDict
 
 import pytz
-
-from Crypto.Signature import PKCS1_v1_5
-from Crypto.PublicKey import RSA
-from Crypto.Hash import SHA
-
-import six
-from six.moves.urllib import parse as urlparse
+import requests
+from Cryptodome.Hash import SHA
+from Cryptodome.PublicKey import RSA
+from Cryptodome.Signature import PKCS1_v1_5
 from six.moves.urllib import parse as urllib
+from six.moves.urllib import parse as urlparse
 
-import base64
-import warnings
-
-from .common import (PaymentCommon, PaymentResponse, FORM, PAID, CANCELLED,
-                     DENIED, ERROR, Form, ResponseError, force_text,
-                     force_byte, _)
 from . import cb
+from .common import (
+    CANCELLED,
+    DENIED,
+    ERROR,
+    FORM,
+    PAID,
+    Form,
+    PaymentCommon,
+    PaymentResponse,
+    ResponseError,
+    _,
+    force_byte,
+    force_text,
+)
 
 __all__ = ['sign', 'Payment']
 
@@ -112,9 +118,7 @@ PAYBOX_ERROR_CODES = {
         'message': 'Opération sans authentification 3-DSecure, bloquée par le filtre.',
         'result': DENIED,
     },
-    '99999': {
-        'message': 'Opération en attente de validation par l’émetteur du moyen de paiement.'
-    },
+    '99999': {'message': 'Opération en attente de validation par l’émetteur du moyen de paiement.'},
 }
 
 ALGOS = {
@@ -125,18 +129,15 @@ ALGOS = {
 }
 
 URLS = {
-    'test':
-        'https://preprod-tpeweb.paybox.com/cgi/MYchoix_pagepaiement.cgi',
-    'prod':
-        'https://tpeweb.paybox.com/cgi/MYchoix_pagepaiement.cgi',
-    'backup':
-        'https://tpeweb1.paybox.com/cgi/MYchoix_pagepaiement.cgi',
+    'test': 'https://preprod-tpeweb.paybox.com/cgi/MYchoix_pagepaiement.cgi',
+    'prod': 'https://tpeweb.paybox.com/cgi/MYchoix_pagepaiement.cgi',
+    'backup': 'https://tpeweb1.paybox.com/cgi/MYchoix_pagepaiement.cgi',
 }
 
 PAYBOX_DIRECT_URLS = {
     'test': 'https://preprod-ppps.paybox.com/PPPS.php',
     'prod': 'https://ppps.paybox.com/PPPS.php',
-    'backup': 'https://ppps1.paybox.com/PPPS.php'
+    'backup': 'https://ppps1.paybox.com/PPPS.php',
 }
 
 PAYBOX_DIRECT_CANCEL_OPERATION = '00055'
@@ -147,14 +148,13 @@ PAYBOX_DIRECT_VERSION_NUMBER = '00103'
 PAYBOX_DIRECT_SUCCESS_RESPONSE_CODE = '00000'
 
 # payment modes
-PAYMENT_MODES = {'AUTHOR_CAPTURE': 'O',
-                 'IMMEDIATE': 'N'}
+PAYMENT_MODES = {'AUTHOR_CAPTURE': 'O', 'IMMEDIATE': 'N'}
 
 
 def sign(data, key):
-    '''Take a list of tuple key, value and sign it by building a string to
-       sign.
-    '''
+    """Take a list of tuple key, value and sign it by building a string to
+    sign.
+    """
     logger = logging.getLogger(__name__)
     algo = None
     logger.debug('signature key %r', key)
@@ -181,20 +181,21 @@ def verify(data, signature, key=PAYBOX_KEY):
 
 
 class Payment(PaymentCommon):
-    '''Paybox backend for eopayment.
+    """Paybox backend for eopayment.
 
-       If you want to handle Instant Payment Notification, you must pass
-       provide a automatic_return_url option specifying the URL of the
-       callback endpoint.
+    If you want to handle Instant Payment Notification, you must pass
+    provide a automatic_return_url option specifying the URL of the
+    callback endpoint.
 
-       Email is mandatory to emit payment requests with paybox.
+    Email is mandatory to emit payment requests with paybox.
+
+    IP adresses to authorize:
+                              IN           OUT
+        test           195.101.99.73 195.101.99.76
+        production     194.2.160.66  194.2.122.158
+        backup         195.25.7.146  195.25.7.166
+    """
 
-       IP adresses to authorize:
-                                 IN           OUT
-           test           195.101.99.73 195.101.99.76
-           production     194.2.160.66  194.2.122.158
-           backup         195.25.7.146  195.25.7.166
-    '''
     callback = None
 
     description = {
@@ -219,49 +220,46 @@ class Payment(PaymentCommon):
                     ('test', 'Test'),
                     ('backup', 'Backup'),
                     ('prod', 'Production'),
-                )
+                ),
             },
             {
                 'name': 'site',
                 'caption': 'Numéro de site',
                 'required': True,
-                'validation': lambda x: isinstance(x, six.string_types)
-                and x.isdigit() and len(x) == 7,
+                'validation': lambda x: isinstance(x, str) and x.isdigit() and len(x) == 7,
             },
             {
                 'name': 'cle',
-                'caption': _('Site key'),
+                'caption': 'Clé',
+                'help_text': 'Uniquement nécessaire pour l\'annulation / remboursement / encaissement (PayBox Direct)',
                 'required': False,
-                'validation': lambda x: isinstance(x, six.string_types),
+                'validation': lambda x: isinstance(x, str),
             },
             {
                 'name': 'rang',
                 'caption': 'Numéro de rang',
                 'required': True,
-                'validation': lambda x: isinstance(x, six.string_types)
-                and x.isdigit() and len(x) == 3,
+                'validation': lambda x: isinstance(x, str) and x.isdigit() and len(x) == 3,
             },
             {
                 'name': 'identifiant',
                 'caption': 'Identifiant',
                 'required': True,
-                'validation': lambda x: isinstance(x, six.string_types)
-                and x.isdigit() and (0 < len(x) < 10),
+                'validation': lambda x: isinstance(x, str) and x.isdigit() and (0 < len(x) < 10),
             },
             {
                 'name': 'shared_secret',
                 'caption': 'Secret partagé (clé HMAC)',
                 'validation': lambda x: isinstance(x, str)
-                and all(a.lower() in '0123456789abcdef' for a in x),
+                and all(a.lower() in '0123456789abcdef' for a in x)
+                and (len(x) % 2 == 0),
                 'required': True,
             },
             {
                 'name': 'devise',
                 'caption': 'Devise',
                 'default': '978',
-                'choices': (
-                    ('978', 'Euro'),
-                ),
+                'choices': (('978', 'Euro'),),
             },
             {
                 'name': 'callback',
@@ -273,30 +271,29 @@ class Payment(PaymentCommon):
                 'caption': 'Nombre de jours pour un paiement différé',
                 'default': '',
                 'required': False,
-                'validation': lambda x: isinstance(x, six.string_types)
-                and x.isdigit() and (1 <= len(x) <= 2)
+                'validation': lambda x: isinstance(x, str) and x.isdigit() and (1 <= len(x) <= 2),
             },
             {
                 'name': 'capture_mode',
                 'caption': _('Capture Mode'),
                 'default': 'IMMEDIATE',
                 'required': False,
-                'choices': list(PAYMENT_MODES)
+                'choices': list(PAYMENT_MODES),
             },
             {
                 'name': 'manual_validation',
                 'caption': 'Validation manuelle',
                 'type': bool,
                 'default': False,
-                'scope': 'transaction'
+                'scope': 'transaction',
             },
             {
                 'name': 'timezone',
                 'caption': _('Default Timezone'),
                 'default': 'Europe/Paris',
                 'required': False,
-            }
-        ]
+            },
+        ],
     }
 
     def make_pbx_cmd(self, guid, orderid=None, transaction_id=None):
@@ -316,9 +313,9 @@ class Payment(PaymentCommon):
         d['PBX_TOTAL'] = self.clean_amount(amount)
         d['PBX_DEVISE'] = force_text(self.devise)
         guid = str(uuid.uuid4().hex)
-        transaction_id = d['PBX_CMD'] = self.make_pbx_cmd(guid=guid,
-                                                          transaction_id=kwargs.get('transaction_id'),
-                                                          orderid=orderid)
+        transaction_id = d['PBX_CMD'] = self.make_pbx_cmd(
+            guid=guid, transaction_id=kwargs.get('transaction_id'), orderid=orderid
+        )
         d['PBX_PORTEUR'] = force_text(email)
         d['PBX_RETOUR'] = (
             'montant:M;reference:R;code_autorisation:A;erreur:E;numero_appel:T;'
@@ -328,8 +325,8 @@ class Payment(PaymentCommon):
         )
         d['PBX_HASH'] = 'SHA512'
         d['PBX_TIME'] = kwargs.get('time') or (
-            force_text(datetime.datetime.utcnow().isoformat('T')).split('.')[0]
-            + '+00:00')
+            force_text(datetime.datetime.utcnow().isoformat('T')).split('.')[0] + '+00:00'
+        )
         d['PBX_ARCHIVAGE'] = orderid or guid
         if self.normal_return_url:
             d['PBX_EFFECTUE'] = self.normal_return_url
@@ -338,8 +335,7 @@ class Payment(PaymentCommon):
             d['PBX_ATTENTE'] = self.normal_return_url
         automatic_return_url = self.automatic_return_url
         if not automatic_return_url and self.callback:
-            warnings.warn("callback option is deprecated, "
-                          "use automatic_return_url", DeprecationWarning)
+            warnings.warn("callback option is deprecated, " "use automatic_return_url", DeprecationWarning)
             automatic_return_url = self.callback
         capture_day = capture_day = kwargs.get('capture_day', self.capture_day)
         if capture_day:
@@ -351,36 +347,44 @@ class Payment(PaymentCommon):
             d['PBX_REPONDRE_A'] = force_text(automatic_return_url)
         d = d.items()
 
-        if six.PY3:
-            shared_secret = codecs.decode(bytes(self.shared_secret, 'ascii'), 'hex')
-        else:
-            shared_secret = codecs.decode(bytes(self.shared_secret), 'hex')
+        shared_secret = codecs.decode(bytes(self.shared_secret, 'ascii'), 'hex')
         d = sign(d, shared_secret)
         url = URLS[self.platform]
         fields = []
         for k, v in d:
-            fields.append({
-                'type': u'hidden',
-                'name': force_text(k),
-                'value': force_text(v),
-            })
-        form = Form(url, 'POST', fields, submit_name=None,
-                    submit_value=u'Envoyer', encoding='utf-8')
+            fields.append(
+                {
+                    'type': 'hidden',
+                    'name': force_text(k),
+                    'value': force_text(v),
+                }
+            )
+        form = Form(url, 'POST', fields, submit_name=None, submit_value='Envoyer', encoding='utf-8')
         return transaction_id, FORM, form
 
     def response(self, query_string, callback=False, **kwargs):
         d = urlparse.parse_qs(query_string, True, False)
-        if not set(d) >= set(['erreur', 'reference']):
+        if not set(d) >= {'erreur', 'reference'}:
             raise ResponseError('missing erreur or reference')
         signed = False
         if 'signature' in d:
             sig = d['signature'][0]
-            sig = base64.b64decode(sig)
+            try:
+                sig = base64.b64decode(sig)
+            except (TypeError, ValueError):
+                raise ResponseError('invalid signature')
             data = []
             if callback:
-                for key in ('montant', 'reference', 'code_autorisation',
-                            'erreur', 'numero_appel', 'numero_transaction',
-                            'date_transaction', 'heure_transaction'):
+                for key in (
+                    'montant',
+                    'reference',
+                    'code_autorisation',
+                    'erreur',
+                    'numero_appel',
+                    'numero_transaction',
+                    'date_transaction',
+                    'heure_transaction',
+                ):
                     data.append('%s=%s' % (key, urllib.quote(d[key][0])))
             else:
                 for key, value in urlparse.parse_qsl(query_string, True, True):
@@ -418,7 +422,8 @@ class Payment(PaymentCommon):
             bank_data=d,
             result=result,
             bank_status=message,
-            transaction_date=transaction_date)
+            transaction_date=transaction_date,
+        )
 
     def perform(self, amount, bank_data, operation):
         logger = logging.getLogger(__name__)
@@ -442,8 +447,6 @@ class Payment(PaymentCommon):
         logger.debug('received %r', response.text)
         data = dict(urlparse.parse_qsl(response.text, True, True))
         if data.get('CODEREPONSE') != PAYBOX_DIRECT_SUCCESS_RESPONSE_CODE:
-            if six.PY2:
-                raise ResponseError(data['COMMENTAIRE'].encode('utf-8'))
             raise ResponseError(data['COMMENTAIRE'])
         return data
 
