@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # eopayment - online payment library
 # Copyright (C) 2011-2020 Entr'ouvert
 #
@@ -16,20 +15,32 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import base64
-import pytz
 import datetime as dt
 import hashlib
 import hmac
 import random
 import re
 import string
-import six
-from six.moves.urllib import parse as urlparse
 import warnings
 
-from .common import (PaymentCommon, PaymentResponse, PAID, DENIED, CANCELLED,
-                     ERROR, FORM, Form, ResponseError, force_text, force_byte, _)
+import pytz
+from six.moves.urllib import parse as urlparse
+
 from .cb import translate_cb_error_code
+from .common import (
+    CANCELLED,
+    DENIED,
+    ERROR,
+    FORM,
+    PAID,
+    Form,
+    PaymentCommon,
+    PaymentResponse,
+    ResponseError,
+    _,
+    force_byte,
+    force_text,
+)
 
 __all__ = ['Payment']
 
@@ -57,8 +68,7 @@ VADS_EOPAYMENT_TRANS_ID = 'vads_ext_info_eopayment_trans_id'
 
 
 def isonow():
-    return dt.datetime.utcnow().isoformat('T').replace('-', '') \
-        .replace('T', '').replace(':', '')[:14]
+    return dt.datetime.utcnow().isoformat('T').replace('-', '').replace('T', '').replace(':', '')[:14]
 
 
 def parse_utc(value):
@@ -70,9 +80,19 @@ def parse_utc(value):
 
 
 class Parameter:
-    def __init__(self, name, ptype, code, max_length=None, length=None,
-                 needed=False, default=None, choices=None, description=None,
-                 help_text=None):
+    def __init__(
+        self,
+        name,
+        ptype,
+        code,
+        max_length=None,
+        length=None,
+        needed=False,
+        default=None,
+        choices=None,
+        description=None,
+        help_text=None,
+    ):
         self.name = name
         self.ptype = ptype
         self.code = code
@@ -107,8 +127,9 @@ class Parameter:
 
 PARAMETERS = [
     # amount as euro cents
-    Parameter('vads_action_mode', None, 47, needed=True, default='INTERACTIVE',
-              choices=('SILENT', 'INTERACTIVE')),
+    Parameter(
+        'vads_action_mode', None, 47, needed=True, default='INTERACTIVE', choices=('SILENT', 'INTERACTIVE')
+    ),
     Parameter('vads_amount', 'n', 9, max_length=12, needed=True),
     Parameter('vads_capture_delay', 'n', 6, max_length=3, default=''),
     # Same as 'vads_capture_delay' but matches other backend naming for
@@ -127,44 +148,38 @@ PARAMETERS = [
     Parameter('vads_cust_title', 'an', 17, max_length=63),
     Parameter('vads_cust_city', 'an', 21, max_length=63),
     Parameter('vads_cust_zip', 'an', 20, max_length=63),
-    Parameter('vads_ctx_mode', 'a', 11, needed=True, choices=('TEST',
-                                                              'PRODUCTION'),
-              default='TEST'),
+    Parameter('vads_ctx_mode', 'a', 11, needed=True, choices=('TEST', 'PRODUCTION'), default='TEST'),
     # ISO 639 code
     Parameter('vads_language', 'a', 12, length=2, default='fr'),
     Parameter('vads_order_id', 'an-', 13, max_length=32),
-    Parameter('vads_order_info', 'an', 14, max_length=255,
-              description="Complément d'information 1"),
-    Parameter('vads_order_info2', 'an', 14, max_length=255,
-              description="Complément d'information 2"),
-    Parameter('vads_order_info3', 'an', 14, max_length=255,
-              description="Complément d'information 3"),
-    Parameter('vads_page_action', None, 46, needed=True, default='PAYMENT',
-              choices=('PAYMENT',)),
-    Parameter('vads_payment_cards', 'an;', 8, max_length=127, default='',
-              description='Liste des cartes de paiement acceptées',
-              help_text='vide ou des valeurs sépareés par un point-virgule '
-                        'parmi AMEX, AURORE-MULTI, BUYSTER, CB, COFINOGA, '
-                        'E-CARTEBLEUE, MASTERCARD, JCB, MAESTRO, ONEY, '
-                        'ONEY_SANDBOX, PAYPAL, PAYPAL_SB, PAYSAFECARD, '
-                        'VISA'),
+    Parameter('vads_order_info', 'an', 14, max_length=255, description="Complément d'information 1"),
+    Parameter('vads_order_info2', 'an', 14, max_length=255, description="Complément d'information 2"),
+    Parameter('vads_order_info3', 'an', 14, max_length=255, description="Complément d'information 3"),
+    Parameter('vads_page_action', None, 46, needed=True, default='PAYMENT', choices=('PAYMENT',)),
+    Parameter(
+        'vads_payment_cards',
+        'an;',
+        8,
+        max_length=127,
+        default='',
+        description='Liste des cartes de paiement acceptées',
+        help_text='vide ou des valeurs sépareés par un point-virgule '
+        'parmi AMEX, AURORE-MULTI, BUYSTER, CB, COFINOGA, '
+        'E-CARTEBLEUE, MASTERCARD, JCB, MAESTRO, ONEY, '
+        'ONEY_SANDBOX, PAYPAL, PAYPAL_SB, PAYSAFECARD, '
+        'VISA',
+    ),
     # must be SINGLE or MULTI with parameters
-    Parameter('vads_payment_config', '', 7, default='SINGLE',
-              choices=('SINGLE', 'MULTI'), needed=True),
-    Parameter('vads_return_mode', None, 48, default='GET',
-              choices=('', 'NONE', 'POST', 'GET')),
+    Parameter('vads_payment_config', '', 7, default='SINGLE', choices=('SINGLE', 'MULTI'), needed=True),
+    Parameter('vads_return_mode', None, 48, default='GET', choices=('', 'NONE', 'POST', 'GET')),
     Parameter('signature', 'an', None, length=40),
-    Parameter('vads_site_id', 'n', 2, length=8, needed=True,
-              description='Identifiant de la boutique'),
+    Parameter('vads_site_id', 'n', 2, length=8, needed=True, description='Identifiant de la boutique'),
     Parameter('vads_theme_config', 'ans', 32, max_length=255),
-    Parameter(VADS_TRANS_DATE, 'n', 4, length=14, needed=True,
-              default=isonow),
+    Parameter(VADS_TRANS_DATE, 'n', 4, length=14, needed=True, default=isonow),
     # https://paiement.systempay.fr/doc/fr-FR/form-payment/reference/vads-trans-id.html
     Parameter('vads_trans_id', 'an', 3, length=6, needed=True),
-    Parameter('vads_validation_mode', 'n', 5, max_length=1, choices=('', '0', '1'),
-              default=''),
-    Parameter('vads_version', 'an', 1, default='V2', needed=True,
-              choices=('V2',)),
+    Parameter('vads_validation_mode', 'n', 5, max_length=1, choices=('', '0', '1'), default=''),
+    Parameter('vads_version', 'an', 1, default='V2', needed=True, choices=('V2',)),
     Parameter('vads_url_success', 'ans', 24, max_length=1024),
     Parameter('vads_url_referral', 'ans', 26, max_length=127),
     Parameter('vads_url_refused', 'ans', 25, max_length=1024),
@@ -176,8 +191,7 @@ PARAMETERS = [
     Parameter(VADS_CUST_FIRST_NAME, 'ans', 104, max_length=63),
     Parameter(VADS_CUST_LAST_NAME, 'ans', 104, max_length=63),
 ]
-PARAMETER_MAP = dict(((parameter.name,
-                       parameter) for parameter in PARAMETERS))
+PARAMETER_MAP = {parameter.name: parameter for parameter in PARAMETERS}
 
 
 def add_vads(kwargs):
@@ -196,33 +210,34 @@ def check_vads(kwargs, exclude=[]):
         if name not in kwargs and name not in exclude and parameter.needed:
             raise ValueError('parameter %s must be defined' % name)
         if name in kwargs and not parameter.check_value(kwargs[name]):
-            raise ValueError('parameter %s value %s is not of the type %s' % (
-                name, kwargs[name],
-                parameter.ptype))
+            raise ValueError(
+                'parameter %s value %s is not of the type %s' % (name, kwargs[name], parameter.ptype)
+            )
 
 
 class Payment(PaymentCommon):
-    '''
-        Produce request for and verify response from the SystemPay payment
-        gateway.
+    """
+    Produce request for and verify response from the SystemPay payment
+    gateway.
 
-            >>> gw =Payment(dict(secret_test='xxx', secret_production='yyyy',
-                                 site_id=123, ctx_mode='PRODUCTION'))
-            >>> print gw.request(100)
-            ('20120525093304_188620',
-            'https://paiement.systempay.fr/vads-payment/?vads_url_return=http%3A%2F%2Furl.de.retour%2Fretour.php&vads_cust_country=FR&vads_site_id=&vads_payment_config=SINGLE&vads_trans_id=188620&vads_action_mode=INTERACTIVE&vads_contrib=eopayment&vads_page_action=PAYMENT&vads_trans_date=20120525093304&vads_ctx_mode=TEST&vads_validation_mode=&vads_version=V2&vads_payment_cards=&signature=5d412498ab523627ec5730a09118f75afa602af5&vads_language=fr&vads_capture_delay=&vads_currency=978&vads_amount=100&vads_return_mode=NONE',
-            {'vads_url_return': 'http://url.de.retour/retour.php',
-            'vads_cust_country': 'FR', 'vads_site_id': '',
-            'vads_payment_config': 'SINGLE', 'vads_trans_id': '188620',
-            'vads_action_mode': 'INTERACTIVE', 'vads_contrib': 'eopayment',
-            'vads_page_action': 'PAYMENT', 'vads_trans_date': '20120525093304',
-            'vads_ctx_mode': 'TEST', 'vads_validation_mode': '',
-            'vads_version': 'V2', 'vads_payment_cards': '', 'signature':
-            '5d412498ab523627ec5730a09118f75afa602af5', 'vads_language': 'fr',
-            'vads_capture_delay': '', 'vads_currency': '978', 'vads_amount': 100,
-            'vads_return_mode': 'NONE'})
+        >>> gw =Payment(dict(secret_test='xxx', secret_production='yyyy',
+                             site_id=123, ctx_mode='PRODUCTION'))
+        >>> print gw.request(100)
+        ('20120525093304_188620',
+        'https://paiement.systempay.fr/vads-payment/?vads_url_return=http%3A%2F%2Furl.de.retour%2Fretour.php&vads_cust_country=FR&vads_site_id=&vads_payment_config=SINGLE&vads_trans_id=188620&vads_action_mode=INTERACTIVE&vads_contrib=eopayment&vads_page_action=PAYMENT&vads_trans_date=20120525093304&vads_ctx_mode=TEST&vads_validation_mode=&vads_version=V2&vads_payment_cards=&signature=5d412498ab523627ec5730a09118f75afa602af5&vads_language=fr&vads_capture_delay=&vads_currency=978&vads_amount=100&vads_return_mode=NONE',
+        {'vads_url_return': 'http://url.de.retour/retour.php',
+        'vads_cust_country': 'FR', 'vads_site_id': '',
+        'vads_payment_config': 'SINGLE', 'vads_trans_id': '188620',
+        'vads_action_mode': 'INTERACTIVE', 'vads_contrib': 'eopayment',
+        'vads_page_action': 'PAYMENT', 'vads_trans_date': '20120525093304',
+        'vads_ctx_mode': 'TEST', 'vads_validation_mode': '',
+        'vads_version': 'V2', 'vads_payment_cards': '', 'signature':
+        '5d412498ab523627ec5730a09118f75afa602af5', 'vads_language': 'fr',
+        'vads_capture_delay': '', 'vads_currency': '978', 'vads_amount': 100,
+        'vads_return_mode': 'NONE'})
+
+    """
 
-    '''
     has_free_transaction_id = True
     service_url = "https://paiement.systempay.fr/vads-payment/"
     signature_algo = 'sha1'
@@ -241,57 +256,75 @@ class Payment(PaymentCommon):
                 'caption': _('Automatic return URL (ignored, must be set in Payzen/SystemPay backoffice)'),
                 'required': False,
             },
-            {'name': 'service_url',
+            {
+                'name': 'service_url',
                 'default': service_url,
                 'caption': 'URL du service de paiement',
                 'help_text': 'ne pas modifier si vous ne savez pas',
                 'validation': lambda x: x.startswith('http'),
-                'required': True, },
-            {'name': 'secret_test',
+                'required': True,
+            },
+            {
+                'name': 'secret_test',
                 'caption': 'Secret pour la configuration de TEST',
                 'validation': lambda value: str.isalnum(value),
-                'required': True, },
-            {'name': 'secret_production',
+                'required': True,
+            },
+            {
+                'name': 'secret_production',
                 'caption': 'Secret pour la configuration de PRODUCTION',
-                'validation': lambda value: str.isalnum(value), },
-            {'name': 'signature_algo',
+                'validation': lambda value: str.isalnum(value),
+            },
+            {
+                'name': 'signature_algo',
                 'caption': 'Algorithme de signature',
                 'default': 'sha1',
                 'choices': (
                     ('sha1', 'SHA-1'),
                     ('hmac_sha256', 'HMAC-SHA-256'),
-                )},
+                ),
+            },
             {
                 'name': 'manual_validation',
                 'caption': 'Validation manuelle',
                 'type': bool,
                 'default': False,
-                'scope': 'transaction'
-            }
-        ]
+                'scope': 'transaction',
+            },
+        ],
     }
 
-    for name in ('vads_ctx_mode', VADS_SITE_ID, 'vads_order_info',
-                 'vads_order_info2', 'vads_order_info3',
-                 'vads_payment_cards', 'vads_payment_config', 'capture_day'):
+    for name in (
+        'vads_ctx_mode',
+        VADS_SITE_ID,
+        'vads_order_info',
+        'vads_order_info2',
+        'vads_order_info3',
+        'vads_payment_cards',
+        'vads_payment_config',
+        'capture_day',
+    ):
         parameter = PARAMETER_MAP[name]
 
         def check_value(parameter):
             def validate(value):
                 return parameter.check_value(value)
+
             return validate
 
-        x = {'name': name,
-             'caption': parameter.description or name,
-             'validation': check_value(parameter),
-             'default': parameter.default,
-             'required': parameter.needed,
-             'help_text': parameter.help_text,
-             'max_length': parameter.max_length}
+        x = {
+            'name': name,
+            'caption': parameter.description or name,
+            'validation': check_value(parameter),
+            'default': parameter.default,
+            'required': parameter.needed,
+            'help_text': parameter.help_text,
+            'max_length': parameter.max_length,
+        }
         description['parameters'].append(x)
 
     def __init__(self, options, logger=None):
-        super(Payment, self).__init__(options, logger=logger)
+        super().__init__(options, logger=logger)
         options = add_vads(options)
         self.options = options
 
@@ -300,28 +333,48 @@ class Payment(PaymentCommon):
         # trans_id starting with 9 are reserved for the systempay backoffice
         # https://paiement.systempay.fr/doc/fr-FR/form-payment/reference/vads-trans-id.html
         gen = random.SystemRandom()
-        if six.PY3:
-            alphabet = string.ascii_letters + string.digits
-        else:
-            alphabet = string.letters + string.digits
+
+        alphabet = string.ascii_letters + string.digits
         first_letter_alphabet = alphabet.replace('9', '')
-        vads_trans_id = (
-            gen.choice(first_letter_alphabet)
-            + ''.join(gen.choice(alphabet) for i in range(5))
-        )
+        vads_trans_id = gen.choice(first_letter_alphabet) + ''.join(gen.choice(alphabet) for i in range(5))
         return vads_trans_id
 
-    def request(self, amount, name=None, first_name=None, last_name=None,
-                address=None, email=None, phone=None, orderid=None, info1=None,
-                info2=None, info3=None, next_url=None, manual_validation=None,
-                transaction_id=None, **kwargs):
-        '''
-           Create the URL string to send a request to SystemPay
-        '''
-        self.logger.debug('%s amount %s name %s address %s email %s phone %s '
-                          'next_url %s info1 %s info2 %s info3 %s kwargs: %s',
-                          __name__, amount, name, address, email, phone, info1,
-                          info2, info3, next_url, kwargs)
+    def request(
+        self,
+        amount,
+        name=None,
+        first_name=None,
+        last_name=None,
+        address=None,
+        email=None,
+        phone=None,
+        orderid=None,
+        info1=None,
+        info2=None,
+        info3=None,
+        next_url=None,
+        manual_validation=None,
+        transaction_id=None,
+        **kwargs,
+    ):
+        """
+        Create the URL string to send a request to SystemPay
+        """
+        self.logger.debug(
+            '%s amount %s name %s address %s email %s phone %s '
+            'next_url %s info1 %s info2 %s info3 %s kwargs: %s',
+            __name__,
+            amount,
+            name,
+            address,
+            email,
+            phone,
+            info1,
+            info2,
+            info3,
+            next_url,
+            kwargs,
+        )
         # amount unit is cents
         amount = '%.0f' % (100 * amount)
         kwargs.update(add_vads({'amount': force_text(amount)}))
@@ -329,8 +382,10 @@ class Payment(PaymentCommon):
             raise ValueError('amount must be an integer >= 0')
         normal_return_url = self.normal_return_url
         if next_url:
-            warnings.warn("passing next_url to request() is deprecated, "
-                          "set normal_return_url in options", DeprecationWarning)
+            warnings.warn(
+                "passing next_url to request() is deprecated, " "set normal_return_url in options",
+                DeprecationWarning,
+            )
             normal_return_url = next_url
         if normal_return_url:
             kwargs[VADS_URL_RETURN] = force_text(normal_return_url)
@@ -360,8 +415,7 @@ class Payment(PaymentCommon):
             ptype = 'an-'
             p = Parameter(name, ptype, 13, max_length=32)
             if not p.check_value(orderid):
-                raise ValueError(
-                    '%s value %s is not of the type %s' % (name, orderid, ptype))
+                raise ValueError('%s value %s is not of the type %s' % (name, orderid, ptype))
             kwargs[name] = orderid
 
         vads_trans_id = self.make_vads_trans_id()
@@ -372,8 +426,7 @@ class Payment(PaymentCommon):
         for parameter in PARAMETERS:
             name = parameter.name
             # import default parameters from configuration
-            if name not in fields \
-                    and name in self.options:
+            if name not in fields and name in self.options:
                 fields[name] = force_text(self.options[name])
             # import default parameters from module
             if name not in fields and parameter.default is not None:
@@ -399,11 +452,13 @@ class Payment(PaymentCommon):
             method='POST',
             fields=[
                 {
-                    'type': u'hidden',
+                    'type': 'hidden',
                     'name': force_text(field_name),
                     'value': force_text(field_value),
                 }
-                for field_name, field_value in fields.items()])
+                for field_name, field_value in fields.items()
+            ],
+        )
         return transaction_id, FORM, form
 
     RESULT_MAP = {
@@ -420,9 +475,11 @@ class Payment(PaymentCommon):
         '00': {'message': 'Tous les contrôles se sont déroulés avec succés.'},
         '02': {'message': 'La carte a dépassé l\'encours autorisé.'},
         '03': {'message': 'La carte appartient à la liste grise du commerçant.'},
-        '04': {'messaǵe': 'Le pays d\'émission de la carte appartient à la liste grise du '
-               'commerçant ou le pays d\'émission de la carte n\'appartient pas à la '
-               'liste blanche du commerçant.'},
+        '04': {
+            'messaǵe': 'Le pays d\'émission de la carte appartient à la liste grise du '
+            'commerçant ou le pays d\'émission de la carte n\'appartient pas à la '
+            'liste blanche du commerçant.'
+        },
         '05': {'message': 'L’adresse IP appartient à la liste grise du marchand.'},
         '06': {'message': 'Le code bin appartient à la liste grise du marchand.'},
         '07': {'message': 'Détection d’une e-carte bleue.'},
@@ -430,8 +487,10 @@ class Payment(PaymentCommon):
         '09': {'message': 'Détection d’une carte commerciale étrangère.'},
         '14': {'message': 'Détection d’une carte à autorisation systématique.'},
         '30': {'message': 'Le pays de l’adresse IP appartient à la liste grise.'},
-        '99': {'message': 'Problème technique recontré par le serveur lors du traitement '
-               'd\'un des contrôles locauxi.'},
+        '99': {
+            'message': 'Problème technique recontré par le serveur lors du traitement '
+            'd\'un des contrôles locauxi.'
+        },
     }
 
     @classmethod
@@ -474,9 +533,8 @@ class Payment(PaymentCommon):
 
     def response(self, query_string, **kwargs):
         fields = urlparse.parse_qs(query_string, True)
-        if not set(fields) >= set([SIGNATURE, VADS_CTX_MODE, VADS_AUTH_RESULT]):
-            raise ResponseError('missing %s, %s or %s' % (SIGNATURE, VADS_CTX_MODE,
-                                                          VADS_AUTH_RESULT))
+        if not set(fields) >= {SIGNATURE, VADS_CTX_MODE, VADS_AUTH_RESULT}:
+            raise ResponseError('missing %s, %s or %s' % (SIGNATURE, VADS_CTX_MODE, VADS_AUTH_RESULT))
         for key, value in fields.items():
             fields[key] = value[0]
         copy = fields.copy()
@@ -485,17 +543,21 @@ class Payment(PaymentCommon):
         self.logger.debug('checking systempay response on: %r', copy)
         signature_result = signature == fields[SIGNATURE]
         if not signature_result:
-            self.logger.debug('signature check: %s <!> %s', signature,
-                              fields[SIGNATURE])
+            self.logger.debug('signature check: %s <!> %s', signature, fields[SIGNATURE])
 
         if not signature_result:
             message += ' signature invalide.'
 
         test = fields[VADS_CTX_MODE] == 'TEST'
-        if VADS_EOPAYMENT_TRANS_ID in fields:
-            transaction_id = fields[VADS_EOPAYMENT_TRANS_ID]
+        vads_eopayment_trans_id = fields.get(VADS_EOPAYMENT_TRANS_ID)
+        vads_trans_date = fields.get(VADS_TRANS_DATE)
+        vads_trans_id = fields.get(VADS_TRANS_ID)
+        if vads_eopayment_trans_id:
+            transaction_id = vads_eopayment_trans_id
+        elif vads_trans_date and vads_trans_id:
+            transaction_id = vads_trans_date + '_' + vads_trans_id
         else:
-            transaction_id = '%s_%s' % (copy[VADS_TRANS_DATE], copy[VADS_TRANS_ID])
+            raise ResponseError('backend error', message)
         # the VADS_AUTH_NUMBER is the number to match payment in bank logs
         copy[self.BANK_ID] = copy.get(VADS_AUTH_NUMBER, '')
         transaction_date = None
@@ -509,7 +571,8 @@ class Payment(PaymentCommon):
             transaction_id=transaction_id,
             bank_status=message,
             transaction_date=transaction_date,
-            test=test)
+            test=test,
+        )
         return response
 
     def sha1_sign(self, secret, signed_data):
@@ -521,15 +584,14 @@ class Payment(PaymentCommon):
 
     def signature(self, fields):
         self.logger.debug('got fields %s to sign' % fields)
-        ordered_keys = sorted(
-            [key for key in fields.keys() if key.startswith('vads_')])
+        ordered_keys = sorted(key for key in fields.keys() if key.startswith('vads_'))
         self.logger.debug('ordered keys %s' % ordered_keys)
         ordered_fields = [force_byte(fields[key]) for key in ordered_keys]
         secret = force_byte(getattr(self, 'secret_%s' % fields['vads_ctx_mode'].lower()))
         signed_data = b'+'.join(ordered_fields)
         signed_data = b'%s+%s' % (signed_data, secret)
-        self.logger.debug(u'generating signature on «%s»', signed_data)
+        self.logger.debug('generating signature on «%s»', signed_data)
         sign_method = getattr(self, '%s_sign' % self.signature_algo)
         sign = sign_method(secret, signed_data)
-        self.logger.debug(u'signature «%s»', sign)
+        self.logger.debug('signature «%s»', sign)
         return force_text(sign)
